@@ -1,24 +1,33 @@
-from bson.errors import InvalidId
 from django.core.exceptions import ValidationError
 from django.utils.encoding import smart_str
+
+from rest_framework import serializers
+
+from bson.errors import InvalidId
 from mongoengine import dereference
 from mongoengine.base.document import BaseDocument
 from mongoengine.document import Document
-from rest_framework import serializers
 from mongoengine.fields import ObjectId
 
 
-class MongoDocumentField(serializers.WritableField):
-    MAX_RECURSION_DEPTH = 5  # default value of depth
+class DocumentField(serializers.Field):
+    """
+    Base field for Mongoengine fields that we can not convert to DRF fields.
+
+    To Contributors:
+        - You can subclass DocumentField to implement custom (de)serialization
+    """
+
+    type_label = 'DocumentField'
 
     def __init__(self, *args, **kwargs):
         try:
             self.model_field = kwargs.pop('model_field')
-            self.depth = kwargs.pop('depth', self.MAX_RECURSION_DEPTH)
+            self.depth = kwargs.pop('depth')
         except KeyError:
             raise ValueError("%s requires 'model_field' kwarg" % self.type_label)
 
-        super(MongoDocumentField, self).__init__(*args, **kwargs)
+        super(DocumentField, self).__init__(*args, **kwargs)
 
     def transform_document(self, document, depth):
         data = {}
@@ -64,18 +73,29 @@ class MongoDocumentField(serializers.WritableField):
         else:
             return smart_str(obj) if isinstance(obj, ObjectId) else obj
 
+    def to_internal_value(self, data):
+        return self.model_field.to_python(data)
 
-class ReferenceField(MongoDocumentField):
+    def to_representation(self, value):
+        return self.transform_object(value, self.depth - 1)
+
+
+class ReferenceField(DocumentField):
+    """
+    For ReferenceField.
+    We always dereference DBRef object before serialization
+    TODO: Maybe support DBRef too?
+    """
 
     type_label = 'ReferenceField'
 
-    def from_native(self, value):
+    def to_internal_value(self, data):
         try:
-            dbref = self.model_field.to_python(value)
+            dbref = self.model_field.to_python(data)
         except InvalidId:
             raise ValidationError(self.error_messages['invalid'])
 
-        instance = dereference.DeReference().__call__([dbref])[0]
+        instance = dereference.DeReference()([dbref])[0]
 
         # Check if dereference was successful
         if not isinstance(instance, Document):
@@ -84,22 +104,22 @@ class ReferenceField(MongoDocumentField):
 
         return instance
 
-    def to_native(self, obj):
-        return self.transform_object(obj, self.depth - 1)
+    def to_representation(self, value):
+        return self.transform_object(value, self.depth - 1)
 
 
-class ListField(MongoDocumentField):
+class ListField(DocumentField):
 
     type_label = 'ListField'
 
-    def from_native(self, value):
-        return self.model_field.to_python(value)
+    def to_internal_value(self, data):
+        return self.model_field.to_python(data)
 
-    def to_native(self, obj):
-        return self.transform_object(obj, self.depth - 1)
+    def to_representation(self, value):
+        return self.transform_object(value, self.depth - 1)
 
 
-class EmbeddedDocumentField(MongoDocumentField):
+class EmbeddedDocumentField(DocumentField):
 
     type_label = 'EmbeddedDocumentField'
 
@@ -111,22 +131,37 @@ class EmbeddedDocumentField(MongoDocumentField):
 
         super(EmbeddedDocumentField, self).__init__(*args, **kwargs)
 
-    def get_default_value(self):
-        return self.to_native(self.default())
-
-    def to_native(self, obj):
-        if obj is None:
+    def to_representation(self, value):
+        if value is None:
             return None
         else:
-            return self.transform_object(obj, self.depth)
+            return self.transform_object(value, self.depth)
 
-    def from_native(self, value):
-        return self.model_field.to_python(value)
+    def to_internal_value(self, data):
+        return self.model_field.to_python(data)
 
 
-class DynamicField(MongoDocumentField):
+class DynamicField(DocumentField):
 
     type_label = 'DynamicField'
 
-    def to_native(self, obj):
-        return self.model_field.to_python(obj)
+    def __init__(self, field_name=None, source=None, *args, **kwargs):
+        super(DynamicField, self).__init__(*args, **kwargs)
+        self.field_name = field_name
+        self.source = source
+        if source:
+            self.source_attrs = self.source.split('.')
+
+    def to_representation(self, value):
+        return self.model_field.to_python(value)
+
+
+class ObjectIdField(serializers.Field):
+
+    type_label = 'ObjectIdField'
+
+    def to_representation(self, value):
+        return smart_str(value)
+
+    def to_internal_value(self, data):
+        return ObjectId(data)
\ No newline at end of file
