@@ -1,9 +1,9 @@
 from rest_framework import mixins
 from rest_framework.viewsets import ViewSetMixin
-from rest_framework_mongoengine.generics import MongoAPIView
+from rest_framework_mongoengine.generics import GenericAPIView
 
 
-class MongoGenericViewSet(ViewSetMixin, MongoAPIView):
+class MongoGenericViewSet(ViewSetMixin, GenericAPIView):
     """
     The MongoGenericViewSet class does not provide any actions by default,
     but does include the base set of generic view behavior, such as
@@ -31,4 +31,4 @@ class ReadOnlyModelViewSet(mixins.RetrieveModelMixin,
     """
     A viewset that provides default `list()` and `retrieve()` actions.
     """
-    pass
\ No newline at end of file
+    pass
